//
//  BallonSlider.m
//

#import <QuartzCore/QuartzCore.h>
#import "BallonSlider.h"



/////////////////////////////////////////////

#pragma mark - 吹き出し

/////////////////////////////////////////////

@interface BallonView : UIView

@property	(nonatomic, strong) UIImage		*ballonImage;
@property	(nonatomic, strong) UILabel		*label;

@end

@implementation BallonView

//------------------------------------------
// 初期化
//------------------------------------------
-(id) initWith:(CGRect)frame
{
    if ((self = [super initWithFrame:frame])) {
		
		self.userInteractionEnabled = NO;
		self.opaque = NO;
		UIColor	*labelColor;
		NSString*	path = [[NSBundle mainBundle] pathForResource:@"baloon_whiteGrade" ofType:@"png"];
		self.ballonImage = [UIImage imageWithContentsOfFile:path];
		labelColor = [UIColor blackColor];
		//ラベル
		self.label = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, frame.size.width, 18)];
		self.label.textAlignment = UITextAlignmentCenter;
		self.label.backgroundColor = [UIColor clearColor];
		self.label.font = [UIFont boldSystemFontOfSize:18];
		self.label.textColor = labelColor;
		self.label.center = CGPointMake(frame.size.width/2, (frame.size.height/2)-12);
		[self addSubview:self.label];
	}
	
	return	self;
}

//------------------------------------------
// 描画
//------------------------------------------
- (void)drawRect:(CGRect)rect {
	
	//影
	CGContextRef context = UIGraphicsGetCurrentContext();
	CGContextSetShadowWithColor(context, CGSizeMake(2, 2), 4, [UIColor colorWithRed:0 green:0 blue:0 alpha:0.75].CGColor);
	[self.ballonImage drawInRect:rect];
}

@end






/////////////////////////////////////////////

#pragma mark - スライダーサム

/////////////////////////////////////////////

@interface SliderThum : UIView

@property	(nonatomic, strong) UIImage		*thumImage;
@property	(nonatomic, strong) BallonView	*baloonView;

-(id) initWithImage:(UIImage*)image;
-(BallonView*) makeBallon;
-(void) removeBallon;
-(void) setBallonLabel:(NSString*)text;		// 吹き出しのテキストを設定する
@end

@implementation SliderThum

//------------------------------------------
// 初期化
//------------------------------------------
-(id) initWithImage:(UIImage*)image
{
	CGRect	frame = CGRectMake(0, 0, image.size.width, image.size.height);
    if ((self = [super initWithFrame:frame])) {
		self.userInteractionEnabled = NO;
		self.thumImage = image;
 		self.opaque = NO;
   }
    return self;
}

//------------------------------------------
// 描画
//------------------------------------------
- (void)drawRect:(CGRect)rect {
	[self.thumImage drawInRect:rect blendMode:kCGBlendModeNormal alpha:1.0];
}

//------------------------------------------
// 吹き出し作る
//------------------------------------------
-(BallonView*) makeBallon
{
	if(self.baloonView==nil){
		CGPoint		centerPt = [self convertPoint:self.center fromView:[self superview]];
		CGRect		bounds = self.bounds;
		
		CGRect		frame = CGRectMake(0, 0, 120, 80);
		self.baloonView  = [[BallonView alloc] initWith:frame];
		self.baloonView.center = CGPointMake(centerPt.x,
											 (bounds.size.height/2)-(self.baloonView.frame.size.height/2));
		//自分のサブビューにする
		[self addSubview:self.baloonView];
	}
	
	return	self.baloonView;
}

//------------------------------------------
// 吹き出し削除
//------------------------------------------
-(void) removeBallon
{
	[self.baloonView removeFromSuperview];
	self.baloonView = nil;
}

//------------------------------------------
// 吹き出しのテキストを設定する
//------------------------------------------
-(void) setBallonLabel:(NSString*)text;
{
	self.baloonView.label.text = text;
}

//------------------------------------------
// 吹き出しのテキストラベル
//------------------------------------------
-(UILabel*) baloonLabel
{
	return	self.baloonView.label;
}

@end






/////////////////////////////////////////////

#pragma mark - スライダーバー

/////////////////////////////////////////////

@interface BallonSliderBar : UIView{
	float			_scale;
	SliderThum		*_thum;
	UIImage			*imageMax;
	UIImage			*imageMin;
	float			value;
	float			maxValue;
	float			minValue;
	BOOL			isEventTracking;
}
@property	(nonatomic, copy) SliderValueChangedBlock	handler;


-(void) setBallonLabel:(NSString*)text;			// 吹き出しのテキストを設定する
-(UILabel*) baloonLabel;						// 吹き出しのテキストラベル
-(float) value;									// スライダーの値を返す（0.0〜1.0）
-(void) setValue:(float)val;					// スライダーの値を設定する（0.0〜1.0）
-(void) valueUpdate;
-(void) thumUpdate;
@end

@implementation BallonSliderBar

//------------------------------------------
// 初期化
//------------------------------------------
-(id) initWithFrame:(CGRect)frame
{
    if ((self = [super initWithFrame:frame])) {
		self.opaque = NO;
		value = 0.0;
		maxValue = 1.0;
		minValue = 0.0;
		
		NSString*	path;
		//スライダー
		UIEdgeInsets cap={6,6,6,6}; // top, left, bottom, right
		path = [[NSBundle mainBundle] pathForResource:@"imageMin_def" ofType:@"png"];
		imageMin = [[UIImage imageWithContentsOfFile:path] resizableImageWithCapInsets:cap];
		
		path = [[NSBundle mainBundle] pathForResource:@"imageMax_def" ofType:@"png"];
		imageMax = [[UIImage imageWithContentsOfFile:path] resizableImageWithCapInsets:cap];
		
		//サム
		path = [[NSBundle mainBundle] pathForResource:@"imageThum_def" ofType:@"png"];
		_thum = [[SliderThum alloc] initWithImage:[UIImage imageWithContentsOfFile:path]];
		CGFloat	offset_y = 0;
		[_thum setCenter:CGPointMake((_thum.frame.size.width/2)-4, (_thum.frame.size.height/2)+offset_y)];
		[self addSubview:_thum];
		
		[self valueUpdate];
	}
	return	self;
}

//------------------------------------------
// 描画
//------------------------------------------
- (void)drawRect:(CGRect)rect {
	
	CGFloat	centerH = _thum.frame.size.height / 2;
	CGRect	minFrame;
	CGRect	maxFrame;
	CGFloat	offset_y = 0;
	maxFrame = CGRectMake(
						  0,
						  centerH-(imageMax.size.height/2)+(offset_y),
						  (rect.size.width)*_scale,
						  imageMax.size.height);
	minFrame = CGRectMake(
						  maxFrame.size.width,
						  centerH-(imageMin.size.height/2)+(offset_y),
						  rect.size.width-maxFrame.size.width,
						  imageMin.size.height);
	[imageMin drawInRect:minFrame blendMode:kCGBlendModeNormal alpha:1.0];
	[imageMax drawInRect:maxFrame blendMode:kCGBlendModeNormal alpha:1.0];
}


//------------------------------------------
// タッチダウン
//------------------------------------------
-(void) touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	NSSet*		allTouchs = [event allTouches];
	UITouch*	touch = [allTouchs anyObject];
	CGPoint		location = [touch locationInView:self];
	
	isEventTracking = CGRectContainsPoint(_thum.frame, location);
	//イベントトラック中
	if(isEventTracking){
		
		[_thum makeBallon];//吹き出しを作る
		
		if(self.handler){
			NSString* text = self.handler(value);
			[self setBallonLabel:text];
		}
	}
	else {
		[self nextResponder];
	}
}

//------------------------------------------
// タッチ移動中
//------------------------------------------
-(void) touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	if(isEventTracking){
		NSSet*		allTouchs = [event allTouches];
		UITouch*	touch = [allTouchs anyObject];
		CGPoint		location = [touch locationInView:self];
		
		//サムの位置変更
		if(location.x < (_thum.frame.size.width/2)-12)
			location.x = (_thum.frame.size.width/2)-12;
		else if(location.x > self.frame.size.width-(_thum.frame.size.width/2)+12)
			location.x = self.frame.size.width-(_thum.frame.size.width/2)+12;
		
		_thum.center = CGPointMake(location.x, _thum.center.y);
		[self valueUpdate];
		
		if(self.handler){
			NSString* text = self.handler(value);
			[self setBallonLabel:text];
		}
	}
}

//------------------------------------------
// タッチアップ
//------------------------------------------
-(void) touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	[_thum removeBallon];//吹き出しを削除
	isEventTracking = NO;
}

//------------------------------------------
// スライダーの位置がら value をアップデート
//------------------------------------------
-(void) valueUpdate
{
	CGPoint	pt = [_thum center];
	CGRect	frame = [self frame];
	
	value  = 1.0 * ((pt.x-(_thum.frame.size.width/2)) / (frame.size.width-_thum.frame.size.width) );
	_scale = 1.0 * (pt.x / frame.size.width );
	if(value > 1.0)
		value = 1.0;
	else if(value < 0) {
		value = 0;
	}

	[self setNeedsDisplay];
}

//------------------------------------------
// value からスライダーの位置をアップデート
//------------------------------------------
-(void) thumUpdate
{
	CGPoint	pt = [_thum center];
	CGRect	frame = [self frame];
	float	w, width = frame.size.width - (_thum.frame.size.width/2);
	
	w = width * value;
	pt.x = w + (_thum.frame.size.width/2) - 12;
	_scale = 1.0 * (pt.x / frame.size.width );
	[_thum setCenter:pt];
	[self setNeedsDisplay];
}

//------------------------------------------
// 吹き出しのテキストを設定する
//------------------------------------------
-(void) setBallonLabel:(NSString*)text;
{
	[_thum baloonLabel].text = text;
}

//------------------------------------------
// 吹き出しのテキストラベル
//------------------------------------------
-(UILabel*) baloonLabel
{
	return	[_thum baloonLabel];
}

//------------------------------------------
// スライダーの値を返す
//------------------------------------------
-(float) value
{
	return	value;
}

//------------------------------------------
// スライダーの値を設定する
//------------------------------------------
-(void) setValue:(float)val
{
	value = val;
	if(value > maxValue){
		value = maxValue;
	}
	if(value < minValue){
		value = minValue;
	}
	[self thumUpdate];
}

@end






/////////////////////////////////////////////

#pragma mark - スライダー

/////////////////////////////////////////////

@interface BallonSlider(){
	BallonSliderBar			*sliderBar;
}
@end

@implementation BallonSlider

//初期化
-(id) initWithFrame:(CGRect)frame
			Handler:(SliderValueChangedBlock)handler
{
	if ((self = [super initWithFrame:frame])) {
		self.backgroundColor = [UIColor clearColor];
		float	y=0;
		CGRect	rect = CGRectMake(0, y, frame.size.width, frame.size.height-y);
		sliderBar = [[BallonSliderBar alloc] initWithFrame:rect];
		sliderBar.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin;
		[self addSubview:sliderBar];
		sliderBar.handler = handler;
	}
	
	return	self;
}

// スライダーの値を返す（0.0〜1.0）
-(float) value
{
	return	[sliderBar value];
}

// スライダーの値を設定する（0.0〜1.0）
-(void) setValue:(float)val
{
	[sliderBar setValue:val];
}

@end

